﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VetClinicBusinessLogic.OfficePackage.HelperEnums;
using VetClinicBusinessLogic.OfficePackage.HelperModels;
using VetClinicDataBaseImplement.Implements;

namespace VetClinicBusinessLogic.OfficePackage
{
	public abstract class AbstractSaveToPdfPharmacist
	{
		public void CreateDoc(PdfInfo info)
		{
			CreatePdf(info);
			CreateParagraph(new PdfParagraph
			{
				Text = info.Title,
				Style =
		   "NormalTitle",
				ParagraphAlignment = PdfParagraphAlignmentType.Center
			});
			CreateParagraph(new PdfParagraph
			{
				Text = $"с { info.DateFrom.ToShortDateString() } по { info.DateTo.ToShortDateString() }", Style
		   = "Normal",
				ParagraphAlignment = PdfParagraphAlignmentType.Center
			});
			CreateTable(new List<string> { "4cm", "4cm", "4cm", "4cm" });
			CreateRow(new PdfRowParameters
			{
				Texts = new List<string> { "Дата", "Название медикамента", "Услуга рекомендации",
"Название визита" },
				Style = "NormalTitle",
				ParagraphAlignment = PdfParagraphAlignmentType.Center
			});
			foreach (var medicine in info.Medicines)
			{
                CreateRow(new PdfRowParameters
                {
                    Texts = new List<string> { "", medicine.MedicineName, "", "" },
                    Style = "NormalTitle",
                    ParagraphAlignment = PdfParagraphAlignmentType.Center
                });
				foreach(var visit in medicine.Visits)
				{
                    CreateRow(new PdfRowParameters
                    {
                        Texts = new List<string> { visit.DateVisit.ToString(), "", "", visit.NameVisit },
                        Style = "Normal",
                        ParagraphAlignment = PdfParagraphAlignmentType.Center
                    });
                }
				foreach (var guidance in medicine.Guidances)
				{
					CreateRow(new PdfRowParameters
					{
						Texts = new List<string> { guidance.Date.ToString(), "", guidance.ServiceName, "" },
						Style = "Normal",
						ParagraphAlignment = PdfParagraphAlignmentType.Center
					});
				}
			}
			SavePdf(info);
		}
		protected abstract void CreatePdf(PdfInfo info);
		protected abstract void CreateParagraph(PdfParagraph paragraph);
		protected abstract void CreateTable(List<string> columns);
		protected abstract void CreateRow(PdfRowParameters rowParameters);
		protected abstract void SavePdf(PdfInfo info);
	}
}
